from rest_framework import viewsets, status
from rest_framework.authentication import TokenAuthentication
from rest_framework.response import Response
from rest_framework.decorators import action
import decimal
from django.db.models import Sum, F, Q, Count
from django.db.models import Value as V
from django.db.models.functions import Concat
from datetime import datetime
from rest_framework.views import APIView

from accounts_shared.models import BusinessClient, ConsumerClient
from dateutil.relativedelta import relativedelta
from django.db.models import Case, Value, When


class BusinessClientsByCityView(APIView):
    authentication_classes = (TokenAuthentication,)

    def get(self, request):
        result = (BusinessClient.objects.annotate(addresses_count=Count('addresses')).filter(addresses_count__gt=0)
                  .values('addresses__city__name')
                  .annotate(name=F('addresses__city__name'), value=Count('addresses__city')).values('name', 'value')
                  .order_by()
                  )
        print(result)

        return Response(result, status=status.HTTP_200_OK)




import django_filters

from accounts_shared.models import (
    BusinessClientContact
)


class BusinessClientContactFilter(django_filters.FilterSet):
    account = django_filters.NumberFilter(
        field_name='account', lookup_expr='exact')

    class Meta:
        model = BusinessClientContact
        fields = ['account']



from enum import Enum
from django.db import models
 

class SexChoice(Enum):
    MALE = "Male"
    FEMALE = "Female"

    @classmethod
    def choices(cls):
        return [(choice.name, choice.value) for choice in cls]


class AddressTypeChoices(Enum):
    BILLING = "Billing"
    SHIPPING = "Shipping"

    @classmethod
    def choices(cls):
        return [(choice.name, choice.value) for choice in cls]


class AccountTypeChoices(models.IntegerChoices):
    CONSUMERCLIENT = 0
    BUSINESSCLIENT = 1
    SUPPLIER = 2
    COMPANY = 3
    CERTIFICATIONENTTIY = 4


from rest_framework import viewsets, status
from rest_framework.authentication import TokenAuthentication
from rest_framework.response import Response
from rest_framework.decorators import action
import decimal
from django.db.models import Sum, F, Q, Count
from django.db.models import Value as V
from django.db.models.functions import Concat
from datetime import datetime
from rest_framework.views import APIView

from accounts_shared.models import ConsumerClient
from dateutil.relativedelta import relativedelta
from django.db.models import Case, Value, When


class ConsumerClientsByAgeView(APIView):
    authentication_classes = (TokenAuthentication,)

    range_ages = (
        {"lookup": "gte", "label": "<17", "age": [18]},
        {"lookup": "range", "label": "18-24", "age": [18, 25]},
        {"lookup": "range", "label": "25-34", "age": [25, 35]},
        {"lookup": "range", "label": "35-44", "age": [35, 45]},
        {"lookup": "range", "label": "45-54", "age": [45, 55]},
        {"lookup": "range", "label": "55-64", "age": [55, 65]},
        {"lookup": "lt", "label": ">65", "age": [65]},
    )

    def get(self, request):
        consumer_clients = ConsumerClient.objects.all()
        aggr_query = {}
        current_date = datetime.now().date()

        for item in self.range_ages:
            age = item.get("age")
            lookup = item.get("lookup")
            label = item.get("label")
            # calculate start_date an end_date
            end_date = current_date - relativedelta(years=age[0])
            start_date = current_date - relativedelta(years=age[-1], days=-1)
            f_value = start_date if len(age) == 1 else (start_date, end_date)
            if lookup == "gte":
                aggr_query[label] = Count(
                    Case(When(dateOfBirth__gte=f_value, then=1)))
            elif lookup == "lt":
                aggr_query[label] = Count(
                    Case(When(dateOfBirth__lt=f_value, then=1)))
            else:
                aggr_query[label] = Count(
                    Case(When(dateOfBirth__range=f_value, then=1)))

        # Aggregate values
        qs_values = ConsumerClient.objects.all().aggregate(**aggr_query)
        result = []
        print(qs_values)
        for key, value in qs_values.items():
            item = {}
            item["name"] = key
            item["value"] = value
            result.append(item)
        return Response(result, status=status.HTTP_200_OK)




LEAD_STATUS = (
    ("assigned", "Assigned"),
    ("in process", "In Process"),
    ("converted", "Converted"),
    ("recycled", "Recycled"),
    ("closed", "Closed"),
)


LEAD_SOURCE = (
    ("call", "Call"),
    ("email", "Email"),
    ("existing customer", "Existing Customer"),
    ("partner", "Partner"),
    ("public relations", "Public Relations"),
    ("compaign", "Campaign"),
    ("other", "Other"),
)

STATUS_CHOICE = (
    ("New", "New"),
    ("Assigned", "Assigned"),
    ("Pending", "Pending"),
    ("Closed", "Closed"),
    ("Rejected", "Rejected"),
    ("Duplicate", "Duplicate"),
)

PRIORITY_CHOICE = (
    ("Low", "Low"),
    ("Normal", "Normal"),
    ("High", "High"),
    ("Urgent", "Urgent"),
)

CASE_TYPE = (("Question", "Question"), ("Incident", "Incident"), ("Problem", "Problem"))

STAGES = (
    ("QUALIFICATION", "QUALIFICATION"),
    ("NEEDS ANALYSIS", "NEEDS ANALYSIS"),
    ("VALUE PROPOSITION", "VALUE PROPOSITION"),
    ("ID.DECISION MAKERS", "ID.DECISION MAKERS"),
    ("PERCEPTION ANALYSIS", "PERCEPTION ANALYSIS"),
    ("PROPOSAL/PRICE QUOTE", "PROPOSAL/PRICE QUOTE"),
    ("NEGOTIATION/REVIEW", "NEGOTIATION/REVIEW"),
    ("CLOSED WON", "CLOSED WON"),
    ("CLOSED LOST", "CLOSED LOST"),
)

SOURCES = (
    ("NONE", "NONE"),
    ("CALL", "CALL"),
    ("EMAIL", " EMAIL"),
    ("EXISTING CUSTOMER", "EXISTING CUSTOMER"),
    ("PARTNER", "PARTNER"),
    ("PUBLIC RELATIONS", "PUBLIC RELATIONS"),
    ("CAMPAIGN", "CAMPAIGN"),
    ("WEBSITE", "WEBSITE"),
    ("OTHER", "OTHER"),
)

BUSINESS_CLIENT_TYPE_CHOICE = (
    ("NONE", "NONE"),
    ("RESELLER", "RESELLER"),
)





# Generated by Django 4.0.5 on 2022-06-24 06:35

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('plans_shared', '0003_erpsubscription_company'),
        ('accounts_shared', '0002_businessclient_creditlimit_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='company',
            name='plan',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='companies', to='plans_shared.plan'),
        ),
    ]



from django.urls import include, path
from rest_framework import routers

from accounts_shared.reports.accounts_reports import ConsumerClientsByAgeView
from accounts_shared.reports.business_clients_reports import BusinessClientsByCityView
from .views import (AddressViewSet, SupplierCategoryViewSet,
                    SupplierViewSet, CountryViewSet, BillingAddressViewSet,
                    BusinessClientViewSet, ConsumerClientViewSet, AccountViewSet,
                    ContactViewSet, CompanyViewSet, ClientViewSet,
                    CertificationEntityViewSet, MyCompanyView,
                    CityViewSet, 
                    ShippingAddressesByAccountList,
                    AddressesByAccountList,
                    ShippingAddressViewSet,
                    BookAnAppointmentSearchCompanyLocationsView,
                    BusinessClientContactViewSet,
                    SupplierContactViewSet,
                    CompanyLocationViewSet, TestAccountViewSet, TestView,
                    # ResllerStoreViewSet,
                    )
router = routers.DefaultRouter()
router.register('address', AddressViewSet)
router.register('supplier', SupplierViewSet)
router.register('supplier-category', SupplierCategoryViewSet)
router.register('countries', CountryViewSet)
router.register('billingAddress', BillingAddressViewSet)
router.register('consumerClient', ConsumerClientViewSet)
router.register('businessClient', BusinessClientViewSet)
router.register('account', AccountViewSet)
router.register('contact', ContactViewSet)
router.register('businessClientContact', BusinessClientContactViewSet)
router.register('supplierContact', SupplierContactViewSet)
router.register('country', CountryViewSet)
router.register('city', CityViewSet)
router.register('companyLocation', CompanyLocationViewSet)
# router.register('resllerStore', ResllerStoreViewSet)
router.register('company', CompanyViewSet)
router.register('client', ClientViewSet, basename='client')
router.register('certificationEntity', CertificationEntityViewSet)
router.register('shippingAddress', ShippingAddressViewSet)
router.register('testAccount', TestAccountViewSet)

urlpatterns = [
    path('', include(router.urls)),
    path('myCompany/', MyCompanyView.as_view()),
    path('shipping_addresses_by_account/', ShippingAddressesByAccountList.as_view()),
    path('addresses_by_account/', AddressesByAccountList.as_view()),
    path('book_an_appointment_search_company_locations/', BookAnAppointmentSearchCompanyLocationsView.as_view()),
    path('report_consumer_clients_by_age/', ConsumerClientsByAgeView.as_view()),
    path('report_business_clients_by_city/', BusinessClientsByCityView.as_view()),
    path('test/', TestView.as_view()),

]



# Generated by Django 4.0.5 on 2022-06-23 13:42

import accounts_shared.enums
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('team_common', '0001_initial'),
        ('team', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('profilePicture', models.ImageField(blank=True, default='defaultImage.jpg', null=True, upload_to='accounts-profile-pictures')),
                ('createdOn', models.DateTimeField(auto_now_add=True)),
                ('isActive', models.BooleanField(default=False)),
                ('balance', models.DecimalField(decimal_places=2, default=0, max_digits=10, null=True)),
                ('notes', models.TextField(blank=True, default='', max_length=1000)),
                ('accountType', models.PositiveSmallIntegerField(blank=True, choices=[(0, 'Consumerclient'), (1, 'Businessclient'), (2, 'Supplier'), (3, 'Company'), (4, 'Certificationenttiy')], null=True)),
                ('name', models.CharField(max_length=60, null=True)),
                ('customerId', models.CharField(blank=True, max_length=200, null=True)),
                ('createdBy', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='accountsCreatedByMe', to='team_common.user')),
            ],
            options={
                'verbose_name': 'Account',
                'verbose_name_plural': 'Accounts',
            },
        ),
        migrations.CreateModel(
            name='Address',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=60, null=True)),
                ('addressLine1', models.CharField(max_length=60)),
                ('addressLine2', models.CharField(blank=True, max_length=60)),
                ('region', models.CharField(max_length=60, verbose_name='State / Province / Region')),
                ('postalCode', models.CharField(max_length=12, verbose_name='ZIP / Postal code')),
                ('account', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='addresses', to='accounts_shared.account')),
            ],
            options={
                'verbose_name': 'Address',
                'verbose_name_plural': 'Addresses',
            },
        ),
        migrations.CreateModel(
            name='City',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=100, null=True)),
                ('code', models.CharField(blank=True, max_length=3, null=True)),
            ],
            options={
                'verbose_name': 'City',
                'verbose_name_plural': 'Cities',
            },
        ),
        migrations.CreateModel(
            name='ClientCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=60)),
            ],
            options={
                'verbose_name': 'Client Category',
                'verbose_name_plural': 'Client Categories',
            },
        ),
        migrations.CreateModel(
            name='Contact',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('firstName', models.CharField(max_length=50)),
                ('lastName', models.CharField(max_length=50)),
                ('email', models.EmailField(max_length=254)),
                ('phoneNumber', models.CharField(max_length=20)),
                ('description', models.TextField(blank=True, null=True)),
                ('createdOn', models.DateTimeField(auto_now_add=True)),
                ('isActive', models.BooleanField(default=False)),
                ('profilePicture', models.ImageField(blank=True, default='../media/defaultImage.jpg', null=True, upload_to='contact-profile-pictures')),
                ('title', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='team_common.title')),
            ],
            options={
                'verbose_name': 'Contact',
                'verbose_name_plural': 'Contacts',
            },
        ),
        migrations.CreateModel(
            name='Country',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=60, null=True)),
                ('code', models.CharField(blank=True, max_length=3, null=True)),
            ],
            options={
                'verbose_name': 'Country',
                'verbose_name_plural': 'Countries',
            },
        ),
        migrations.CreateModel(
            name='Store',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254)),
                ('phoneNumber', models.CharField(max_length=20)),
                ('title', models.CharField(max_length=200)),
                ('image', models.ImageField(blank=True, default='../media/defaultImage.jpg', null=True, upload_to='store-pictures')),
            ],
            options={
                'verbose_name': 'Store',
                'verbose_name_plural': 'Stores',
            },
        ),
        migrations.CreateModel(
            name='SupplierCategory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=60)),
            ],
            options={
                'verbose_name': 'Supplier Category',
                'verbose_name_plural': 'Supplier Categories',
            },
        ),
        migrations.CreateModel(
            name='TestAccount',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('createdOn', models.DateTimeField(auto_now_add=True)),
                ('isActive', models.BooleanField(default=False)),
                ('balance', models.DecimalField(decimal_places=2, default=0, max_digits=10, null=True)),
                ('notes', models.TextField(blank=True, default='', max_length=1000)),
                ('name', models.CharField(max_length=60, null=True)),
            ],
            options={
                'verbose_name': 'Test Account',
                'verbose_name_plural': 'Test Accounts',
            },
        ),
        migrations.CreateModel(
            name='BillingAddress',
            fields=[
                ('address_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='accounts_shared.address')),
                ('addressType', models.CharField(choices=[('BILLING', 'Billing'), ('SHIPPING', 'Shipping')], default=accounts_shared.enums.AddressTypeChoices['BILLING'], max_length=10)),
            ],
            options={
                'verbose_name': 'Billing Address',
                'verbose_name_plural': 'Billing Addresses',
            },
            bases=('accounts_shared.address',),
        ),
        migrations.CreateModel(
            name='BusinessAccount',
            fields=[
                ('account_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='accounts_shared.account')),
                ('email', models.EmailField(max_length=254)),
                ('phoneNumber', models.CharField(max_length=20)),
                ('website', models.URLField(blank=True, max_length=500, null=True)),
            ],
            options={
                'verbose_name': 'Business Account',
                'verbose_name_plural': 'Business Accounts',
            },
            bases=('accounts_shared.account',),
        ),
        migrations.CreateModel(
            name='Company',
            fields=[
                ('account_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='accounts_shared.account')),
                ('email', models.EmailField(blank=True, max_length=254, null=True, unique=True)),
                ('phoneNumber', models.CharField(blank=True, max_length=20, null=True)),
                ('website', models.URLField(blank=True, max_length=500, null=True)),
            ],
            options={
                'verbose_name': 'Company',
                'verbose_name_plural': 'Companies',
            },
            bases=('accounts_shared.account',),
        ),
        migrations.CreateModel(
            name='ShippingAddress',
            fields=[
                ('address_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='accounts_shared.address')),
                ('addressType', models.CharField(choices=[('BILLING', 'Billing'), ('SHIPPING', 'Shipping')], default=accounts_shared.enums.AddressTypeChoices['SHIPPING'], max_length=10)),
            ],
            options={
                'verbose_name': 'Shipping Address',
                'verbose_name_plural': 'Shipping Addresses',
            },
            bases=('accounts_shared.address',),
        ),
        migrations.CreateModel(
            name='Lead',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('firstName', models.CharField(max_length=50)),
                ('lastName', models.CharField(max_length=50)),
                ('email', models.EmailField(max_length=254)),
                ('phoneNumber', models.CharField(blank=True, max_length=50, null=True)),
                ('status', models.CharField(blank=True, choices=[('assigned', 'Assigned'), ('in process', 'In Process'), ('converted', 'Converted'), ('recycled', 'Recycled'), ('closed', 'Closed')], max_length=255, null=True)),
                ('source', models.CharField(blank=True, choices=[('call', 'Call'), ('email', 'Email'), ('existing customer', 'Existing Customer'), ('partner', 'Partner'), ('public relations', 'Public Relations'), ('compaign', 'Campaign'), ('other', 'Other')], max_length=255, null=True)),
                ('website', models.CharField(blank=True, max_length=255, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('opportunityAmount', models.DecimalField(blank=True, decimal_places=2, max_digits=12, null=True)),
                ('createdOn', models.DateTimeField(auto_now_add=True)),
                ('isActive', models.BooleanField(default=False)),
                ('addresses', models.ManyToManyField(related_name='leads', to='accounts_shared.address')),
                ('assignedTo', models.ManyToManyField(related_name='myLeads', to='team.employee')),
                ('createdBy', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='team.employee')),
            ],
            options={
                'verbose_name': 'Lead',
                'verbose_name_plural': 'Leads',
                'ordering': ['-createdOn'],
            },
        ),
        migrations.CreateModel(
            name='CompanyLocation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254)),
                ('phoneNumber', models.CharField(max_length=20)),
                ('title', models.CharField(max_length=200)),
                ('image', models.ImageField(blank=True, default='../media/defaultImage.jpg', null=True, upload_to='company-location-pictures')),
                ('address', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='accounts_shared.address')),
            ],
            options={
                'verbose_name': 'Company Location',
                'verbose_name_plural': 'Company Locations',
            },
        ),
        migrations.AddField(
            model_name='address',
            name='city',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='accounts_shared.city'),
        ),
        migrations.AddField(
            model_name='address',
            name='country',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='accounts_shared.country'),
        ),
        migrations.CreateModel(
            name='BusinessClient',
            fields=[
                ('businessaccount_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='accounts_shared.businessaccount')),
                ('bio', models.TextField(blank=True, default='', max_length=1000)),
                ('isReseller', models.BooleanField(default=False)),
                ('clientCategories', models.ManyToManyField(blank=True, related_name='businessClients', to='accounts_shared.clientcategory')),
            ],
            options={
                'verbose_name': 'Business Client',
                'verbose_name_plural': 'Business Clients',
            },
            bases=('accounts_shared.businessaccount',),
        ),
        migrations.CreateModel(
            name='CertificationEntity',
            fields=[
                ('businessaccount_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='accounts_shared.businessaccount')),
                ('code', models.CharField(max_length=30)),
                ('description', models.TextField(blank=True, default='', max_length=1000)),
            ],
            options={
                'verbose_name': 'CertificationEntity',
                'verbose_name_plural': 'CertificationEntities',
            },
            bases=('accounts_shared.businessaccount',),
        ),
        migrations.CreateModel(
            name='Supplier',
            fields=[
                ('businessaccount_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='accounts_shared.businessaccount')),
                ('description', models.TextField(blank=True, default='', max_length=1000)),
                ('supplierCategories', models.ManyToManyField(blank=True, related_name='suppliers', to='accounts_shared.suppliercategory')),
            ],
            options={
                'verbose_name': 'Supplier',
                'verbose_name_plural': 'Suppliers',
            },
            bases=('accounts_shared.businessaccount',),
        ),
        migrations.AddField(
            model_name='businessaccount',
            name='belongsTo',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='businessAccounts', to='accounts_shared.company'),
        ),
        migrations.CreateModel(
            name='SupplierContact',
            fields=[
                ('contact_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='accounts_shared.contact')),
                ('account', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='contacts', to='accounts_shared.supplier')),
            ],
            options={
                'verbose_name': 'Supplier Contact',
                'verbose_name_plural': 'Supplier Contacts',
            },
            bases=('accounts_shared.contact',),
        ),
        migrations.CreateModel(
            name='ConsumerClient',
            fields=[
                ('account_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='accounts_shared.account')),
                ('email', models.EmailField(max_length=254)),
                ('phoneNumber', models.CharField(max_length=20)),
                ('age', models.PositiveIntegerField(default=0)),
                ('dateOfBirth', models.DateField(blank=True, null=True)),
                ('sex', models.CharField(choices=[('MALE', 'Male'), ('FEMALE', 'Female')], max_length=10)),
                ('belongsTo', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='consumerClients', to='accounts_shared.company')),
                ('clientCategories', models.ManyToManyField(blank=True, related_name='consumerClients', to='accounts_shared.clientcategory')),
                ('user', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='account', to='team_common.user')),
            ],
            options={
                'verbose_name': 'Consumer Client',
                'verbose_name_plural': 'Consumer Clients',
                'unique_together': {('belongsTo', 'email')},
            },
            bases=('accounts_shared.account',),
        ),
        migrations.AddField(
            model_name='company',
            name='businessClient',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='company', to='accounts_shared.businessclient'),
        ),
        migrations.CreateModel(
            name='BusinessClientContact',
            fields=[
                ('contact_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='accounts_shared.contact')),
                ('user', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='contact', to='team_common.user')),
                ('account', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='contacts', to='accounts_shared.businessclient')),
            ],
            options={
                'verbose_name': 'Business Client Contact',
                'verbose_name_plural': 'Business Client Contacts',
            },
            bases=('accounts_shared.contact',),
        ),
    ]



from django.db import models

# from users.enums import UserTypeChoices


class ContactManager(models.Manager):

    def get_queryset(self):
        return super().get_queryset()

    def by_user(self, user):
        user_type = user.userType
        # if user_type == UserTypeChoices.EMPLOYEE or user_type == UserTypeChoices.RETAILER:
        #     return super().get_queryset().get(
        #         user=user)
        # else:
        #     raise Exception("This user type does not have a Contact")


class ConsumerClientManager(models.Manager):

    def get_queryset(self):
        return super().get_queryset()

    def by_user(self, user):
        user_type = user.userType
        # if user_type == UserTypeChoices.CONSUMER:
        #     return super().get_queryset().get(
        #         user=user)
        # else:
        #     raise Exception("User is not a ConsumerClient")



import requests
# from users.models import (CustomUser)
from accounts_shared.filters.filters import BusinessClientContactFilter
from team_common.models import User
from .models import (Supplier, SupplierCategory, ShippingAddress, BillingAddress, Country,
                     ConsumerClient, BusinessClient, Contact, Account, Address, Company,
                     City, CompanyLocation,
                     SupplierContact, BusinessClientContact,
                     CertificationEntity, TestAccount,
                     )
from .serializers import (SupplierSerializer, SupplierCategorySerializer,
                          AddressSerializer,
                          CountrySerializer, BillingAddressSerializer, ConsumerClientSerializer,
                          ConsumerClientReadSerializer, BusinessClientSerializer,
                          BusinessClientReadSerializer, ContactSerializer, ContactReadSerializer,
                          AccountReadSerializer, AccountSerializer, AddressReadSerializer,
                          SupplierReadSerializer, CompanySerializer, CompanyReadSerializer,
                          AllClientSerializer,
                          CitySerializer,
                          CompanyLocationSerializer, CompanyLocationReadSerializer,
                          CertificationEntitySerializer,
                          CertificationEntityReadSerializer,
                          ShippingAddressSerializer,
                          ShippingAddressReadSerializer,
                          SupplierContactSerializer,
                          SupplierContactReadSerializer,
                          BusinessClientContactSerializer,
                          BusinessClientContactReadSerializer,
                          TestAccountSerializer
                          )
from rest_framework import viewsets, status
from rest_framework.authentication import TokenAuthentication
from rest_framework.response import Response
from rest_framework.decorators import action
import itertools
from rest_framework.permissions import AllowAny
from rest_framework.views import APIView
import django_filters
from rest_framework import generics
from django.db.models import Q
from django_filters.views import FilterView


class BookAnAppointmentSearchCompanyLocationsFilterSet(django_filters.FilterSet):
    location = django_filters.CharFilter(method='filter_q')
    # postalCode = django_filters.CharFilter(field_name="address__postalCode")

    def filter_q(self, qs, name, value):
        return qs.filter(
            Q(address__city__name__icontains=value) | Q(address__postalCode=value)
        )

    class Meta:
        model = CompanyLocation
        fields = ['address__city', 'address__postalCode']


class BookAnAppointmentSearchCompanyLocationsView(generics.ListAPIView, FilterView):
    serializer_class = CompanyLocationReadSerializer
    queryset = CompanyLocation.objects.all()
    permission_classes = [AllowAny]
    filterset_class = BookAnAppointmentSearchCompanyLocationsFilterSet


class CompanyLocationFilterSet(django_filters.FilterSet):
    city = django_filters.CharFilter(field_name="address__city__id")

    class Meta:
        model = CompanyLocation
        fields = ['city']


class CompanyLocationViewSet(viewsets.ModelViewSet):
    filterset_class = CompanyLocationFilterSet

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return CompanyLocationReadSerializer
        return CompanyLocationSerializer
    queryset = CompanyLocation.objects.all()
    permission_classes = [AllowAny]


class AccountViewSet(viewsets.ModelViewSet):

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return AccountReadSerializer
        return AccountSerializer
    queryset = Account.objects.all()
    authentication_classes = (TokenAuthentication,)

class CompanyViewSet(viewsets.ModelViewSet):

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return CompanyReadSerializer
        return CompanySerializer
    queryset = Company.objects.all()
    authentication_classes = (TokenAuthentication,)

class MyCompanyView(APIView):

    authentication_classes = []
    permission_classes = []

    def get(self, request):
        company = Company.objects.filter(
            email='henryhamamji@gmail.com')[0]
        serializer = CompanyReadSerializer(company, many=False, context={"request": request})
        return Response(serializer.data, status=status.HTTP_200_OK)

class CountryViewSet(viewsets.ModelViewSet):
    serializer_class = CountrySerializer
    queryset = Country.objects.all()
    authentication_classes = [] 
    permission_classes = [AllowAny]

class CityViewSet(viewsets.ModelViewSet):
    serializer_class = CitySerializer
    queryset = City.objects.all()
    permission_classes = [AllowAny]
    authentication_classes = [] 

class SupplierCategoryViewSet(viewsets.ModelViewSet):
    serializer_class = SupplierCategorySerializer
    queryset = SupplierCategory.objects.all()
    authentication_classes = (TokenAuthentication,)

class BillingAddressViewSet(viewsets.ModelViewSet):
    serializer_class = BillingAddressSerializer
    queryset = BillingAddress.objects.all()
    authentication_classes = (TokenAuthentication,)


class ShippingAddressViewSet(viewsets.ModelViewSet):
    queryset = ShippingAddress.objects.all()
    authentication_classes = (TokenAuthentication,)

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return ShippingAddressReadSerializer
        return ShippingAddressSerializer

class AddressViewSet(viewsets.ModelViewSet):

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return AddressReadSerializer
        return AddressSerializer
    queryset = Address.objects.all()
    authentication_classes = (TokenAuthentication,)
    
    def create(self, request):
        data = request.data
        country_id = data.pop('country', None)
        city_id = data.pop('city', None)
        account_id = data.pop('account')
        address = Address.objects.create(
            **data,
            country_id=country_id,
            city_id=city_id,
            account_id=account_id
        )
        serializer = AddressReadSerializer(address, many=False)
        return Response(serializer.data, status=status.HTTP_201_CREATED)

class ShippingAddressesByAccountList(generics.ListAPIView):

    def get_queryset(self):
        queryset = ShippingAddress.objects.all()
        account_id = self.request.query_params.get('account', None)
        if account_id is not None:
            queryset = queryset.filter(account__id=account_id)
        return queryset

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return ShippingAddressReadSerializer
        return ShippingAddressSerializer
    authentication_classes = (TokenAuthentication,)

class AddressesByAccountList(generics.ListAPIView):

    def get_queryset(self):
        queryset = Address.objects.all()
        account_id = self.request.query_params.get('account', None)
        if account_id is not None:
            queryset = queryset.filter(account__id=account_id)
        return queryset

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return AddressReadSerializer
        return AddressSerializer
    authentication_classes = (TokenAuthentication,)

class CertificationEntityViewSet(viewsets.ModelViewSet):

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return CertificationEntityReadSerializer
        return CertificationEntitySerializer
    queryset = CertificationEntity.objects.all()
    authentication_classes = (TokenAuthentication,)

class SupplierViewSet(viewsets.ModelViewSet):

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return SupplierReadSerializer
        return SupplierSerializer
    queryset = Supplier.objects.all()
    authentication_classes = (TokenAuthentication,)

    def get_country(self, request):
        country_data = request.data['address']['country']
        countryId = country_data['id']
        country = Country.objects.get(id=countryId)
        return country

    def create_address(self, request):
        addressLine1 = request.data['address']['addressLine1']
        region = request.data['address']['region']
        city = request.data['address']['city']
        postalCode = request.data['address']['postalCode']
        country = self.get_country(request)
        address = Address.objects.create(
            addressLine1=addressLine1, region=region, city=city, postalCode=postalCode, country=country)
        return address

    @action(detail=False, methods=['POST'])
    def add_supplier(self, request, pk=None):
        if request.data:
            supplierCategoryData = request.data['supplierCategories']
            supplierCategoryIds = [category['id']
                                   for category in supplierCategoryData]
            supplierCategories = SupplierCategory.objects.filter(
                id__in=supplierCategoryIds)
            companyName = request.data['companyName']
            website = request.data['website']
            contactName = request.data['contactName']
            contactEmail = request.data['contactEmail']
            contactPhoneNumber = request.data['contactPhoneNumber']
            newAddress = self.create_address(request)
            supplier = Supplier.objects.create(address=newAddress, companyName=companyName, contactName=contactName,
                                               contactEmail=contactEmail, website=website, contactPhoneNumber=contactPhoneNumber)
            supplier.supplierCategories.set(supplierCategories)
            supplierSerializer = SupplierSerializer(supplier, many=False)
            response = {'message': 'new supplier added',
                        'result': supplierSerializer.data}
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = {'message': 'you need to provide supplier data'}
            return Response(response, status=status.HTTP_400_BAD_REQUEST)

    @action(detail=True, methods=['PUT'])
    def update_contact_name(self, request, pk=None):
        if request.data:
            supplier = Supplier.objects.get(id=pk)
            supplier.contactName = request.data['contactName']
            supplier.save()
            supplierSerializer = SupplierSerializer(supplier, many=False)
            response = {'message': 'supplier contact name was changed',
                        'result': supplierSerializer.data}
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = {'message': 'you need to provide supplier contact name'}
            return Response(response, status=status.HTTP_400_BAD_REQUEST)

    @action(detail=True, methods=['PUT'])
    def update_contact_email(self, request, pk=None):
        if request.data:
            supplier = Supplier.objects.get(id=pk)
            supplier.contactEmail = request.data['contactEmail']
            supplier.save()
            supplierSerializer = SupplierSerializer(supplier, many=False)
            response = {'message': 'supplier contact email was changed',
                        'result': supplierSerializer.data}
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = {
                'message': 'you need to provide supplier contact email'}
            return Response(response, status=status.HTTP_400_BAD_REQUEST)

    @action(detail=True, methods=['PUT'])
    def update_contact_phone_number(self, request, pk=None):
        if request.data:
            supplier = Supplier.objects.get(id=pk)
            supplier.contactEmail = request.data['contactPhoneNumber']
            supplier.save()
            supplierSerializer = SupplierSerializer(supplier, many=False)
            response = {'message': 'supplier contact phone number was changed',
                        'result': supplierSerializer.data}
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = {
                'message': 'you need to provide supplier contact phone number'}
            return Response(response, status=status.HTTP_400_BAD_REQUEST)

    @action(detail=True, methods=['PUT'])
    def update_supplier_categories(self, request, pk=None):
        if request.data:
            supplier = Supplier.objects.get(id=pk)
            supplier_categories_data = request.data['supplierCategories']
            new_supplier_categories = []
            for supplierCategory in supplier_categories_data:
                new_supplier_category = SupplierCategory.objects.get(
                    id=supplierCategory['id'])
                new_supplier_categories.append(new_supplier_category)
            supplier.supplierCategories.set(new_supplier_categories)
            supplier.save()
            supplierSerializer = SupplierSerializer(supplier, many=False)
            response = {'message': 'supplier categories was changed',
                        'result': supplierSerializer.data}
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = {'message': 'you need to provide supplier categories'}
            return Response(response, status=status.HTTP_400_BAD_REQUEST)

    @action(detail=True, methods=['PUT'])
    def update_company_name(self, request, pk=None):
        if request.data:
            supplier = Supplier.objects.get(id=pk)
            supplier.companyName = request.data['companyName']
            supplier.save()
            supplierSerializer = SupplierSerializer(supplier, many=False)
            response = {'message': 'supplier compnay name was changed',
                        'result': supplierSerializer.data}
            return Response(response, status=status.HTTP_200_OK)
        else:
            response = {'message': 'you need to provide supplier compnay name'}
            return Response(response, status=status.HTTP_400_BAD_REQUEST)


class ConsumerClientViewSet(viewsets.ModelViewSet):

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return ConsumerClientReadSerializer
        return ConsumerClientSerializer
    queryset = ConsumerClient.objects.all()
    authentication_classes = (TokenAuthentication,)

    @action(detail=False, methods=['GET'])
    def by_user(self, request, pk=None):
        client = ConsumerClient.objects.by_user(user=request.user)
        serializer = ConsumerClientReadSerializer(client, many=False)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @action(detail=False, methods=['POST'])
    def by_user_id(self, request, pk=None):
        user_id = request.data.get('user', None)
        user = User.objects.get(id=user_id)
        client = ConsumerClient.objects.by_user(user=user)
        serializer = ConsumerClientReadSerializer(client, many=False)
        return Response(serializer.data, status=status.HTTP_200_OK)

class BusinessClientViewSet(viewsets.ModelViewSet):

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return BusinessClientReadSerializer
        return BusinessClientSerializer
    queryset = BusinessClient.objects.all()
    authentication_classes = (TokenAuthentication,)

class ContactViewSet(viewsets.ModelViewSet):

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return ContactReadSerializer
        return ContactSerializer
    queryset = Contact.objects.all()
    authentication_classes = (TokenAuthentication,)

class SupplierContactViewSet(viewsets.ModelViewSet):

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return SupplierContactReadSerializer
        return SupplierContactSerializer
    queryset = SupplierContact.objects.all()
    authentication_classes = (TokenAuthentication,)

class BusinessClientContactViewSet(viewsets.ModelViewSet):

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return BusinessClientContactReadSerializer
        return BusinessClientContactSerializer
    queryset = BusinessClientContact.objects.all()
    authentication_classes = (TokenAuthentication,)
    filter_class = BusinessClientContactFilter

    @action(detail=False, methods=['GET'])
    def by_user(self, request):
        contact = BusinessClientContact.objects.by_user(user=request.user)
        serializer = ContactReadSerializer(contact, many=False)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @action(detail=False, methods=['POST'])
    def by_user_id(self, request):
        user_id = request.data.get('user', None)
        user = User.objects.get(id=user_id)
        contact = BusinessClientContact.objects.by_user(user=user)
        serializer = ContactReadSerializer(contact, many=False)
        return Response(serializer.data, status=status.HTTP_200_OK)

class ClientViewSet(viewsets.ModelViewSet):
    authentication_classes = (TokenAuthentication,)
    """
    API endpoint that lists all clients (Business & Consumer) clients.
    """

    def list(self, request):
        queryset = list(itertools.chain(ConsumerClient.objects.all(), BusinessClient.objects.all()))
        serializer = AllClientSerializer(queryset, many=True)
        return Response(serializer.data)

class TestView(APIView):
    permission_classes = [AllowAny]
    authentication_classes = []
    # permission_classes = []
    def post(self, request):
        test_header = request.headers
        print("tenant ", test_header)
        response = requests.get("https://joole-api.herokuapp.com/cms/mainTabItem/",
                                headers=test_header)
        print("RES", response.reason)
        if response.status_code != 200:
            return Response({"FAIL": response})
        accounts = response.json()
        return Response({"message": "Got some data!", "data": accounts})

class TestAccountViewSet(viewsets.ModelViewSet):

    def get_serializer_class(self):
        if self.request.method == 'GET':
            return TestAccountSerializer
        return TestAccountSerializer
    queryset = TestAccount.objects.all()


from rest_framework import serializers
from .models import (CompanyLocation, Supplier,
                     SupplierCategory, BusinessClient, ConsumerClient,
                     Address, Country, BusinessAccount, Account,
                     City, ShippingAddress, BillingAddress,
                     Contact, Company, CertificationEntity, Lead,
                     SupplierContact, BusinessClientContact, TestAccount,
                     )
from .enums import AccountTypeChoices
from team_common.serializers import TitleSerializer


class Base64ImageField(serializers.ImageField):
    """
    A Django REST framework field for handling image-uploads through raw post data.
    It uses base64 for encoding and decoding the contents of the file.

    Heavily based on
    https://github.com/tomchristie/django-rest-framework/pull/1268

    Updated for Django REST framework 3.
    """

    def to_internal_value(self, data):
        from django.core.files.base import ContentFile
        import base64
        import six
        import uuid

        # Check if this is a base64 string
        if isinstance(data, six.string_types):
            # Check if the base64 string is in the "data:" format
            if 'data:' in data and ';base64,' in data:
                # Break out the header from the base64 content
                header, data = data.split(';base64,')

            # Try to decode the file. Return validation error if it fails.
            try:
                decoded_file = base64.b64decode(data)
            except TypeError:
                self.fail('invalid_image')

            # Generate file name:
            file_name = str(uuid.uuid4())[:12]  # 12 characters are more than enough.
            # Get the file name extension:
            file_extension = self.get_file_extension(file_name, decoded_file)

            complete_file_name = "%s.%s" % (file_name, file_extension, )

            data = ContentFile(decoded_file, name=complete_file_name)

        return super(Base64ImageField, self).to_internal_value(data)

    def get_file_extension(self, file_name, decoded_file):
        import imghdr

        extension = imghdr.what(file_name, decoded_file)
        extension = "jpg" if extension == "jpeg" else extension
        if extension is None:
            return 'jpg'

        return extension


class ContactSerializer(serializers.ModelSerializer):

    class Meta:
        model = Contact
        fields = ['id', 'firstName', 'lastName', 'email', 'phoneNumber',
                  'description', 'createdOn', 'isActive', 'profilePicture', 'title']


class BusinessClientContactSerializer(ContactSerializer):

    class Meta:
        model = BusinessClientContact
        fields = ContactSerializer.Meta.fields + ['account']


class SupplierContactSerializer(ContactSerializer):

    class Meta:
        model = SupplierContact
        fields = ContactSerializer.Meta.fields + ['account']


class CompanySerializer(serializers.ModelSerializer):

    class Meta:
        model = Company
        fields = ['name', 'website', 'phoneNumber', 'email', 'id', 'profilePicture',
                  'businessClient']


class CountrySerializer(serializers.ModelSerializer):

    class Meta:
        model = Country
        fields = ['name', 'code', 'id']


class CitySerializer(serializers.ModelSerializer):

    class Meta:
        model = City
        fields = ['name', 'code', 'id']


class AddressSerializer(serializers.ModelSerializer):

    class Meta:
        model = Address
        fields = ['id', 'addressLine1', 'addressLine2',
                  'region', 'city', 'postalCode', 'country', 'account', 'name']


class AddressReadSerializer(AddressSerializer):
    country = CountrySerializer(many=False)
    city = CitySerializer(many=False)

    class Meta:
        model = Address
        fields = AddressSerializer.Meta.fields + ['value']


class CompanyReadSerializer(CompanySerializer):
    addresses = AddressReadSerializer(many=True)

    class Meta:
        model = Company
        fields = CompanySerializer.Meta.fields + ['employees', 'addresses']


class AccountSerializer(serializers.ModelSerializer):
    profilePicture = Base64ImageField(
        max_length=None, use_url=True, required=False
    )

    class Meta:
        model = Account
        fields = ['id', 'profilePicture', 'createdBy', 'createdOn',
                  'balance', 'notes', 'accountType', 'isActive', 'addresses', 'name']


class AccountReadSerializer(AccountSerializer):
    addresses = AddressReadSerializer(many=True)

    class Meta:
        model = Account
        fields = AccountSerializer.Meta.fields


class BusinessAccountSerializer(serializers.ModelSerializer):

    class Meta:
        model = BusinessAccount
        fields = ['id', 'email', 'phoneNumber', 'name', 'website', 'belongsTo']


class CertificationEntitySerializer(serializers.ModelSerializer):
    profilePicture = Base64ImageField(
        max_length=None, use_url=True, required=False
    )

    class Meta:
        model = CertificationEntity
        fields = ['id', 'email', 'phoneNumber', 'name', 'website', 'code',
                  'accountType', 'profilePicture', 'description', 'balance', 'isActive']


class ShippingAddressSerializer(AddressSerializer):

    class Meta:
        model = ShippingAddress
        fields = AddressSerializer.Meta.fields


class ShippingAddressReadSerializer(AddressSerializer):
    country = CountrySerializer(many=False)
    city = CitySerializer(many=False)
    addressType = serializers.CharField(source='get_addressType_display')

    class Meta:
        model = ShippingAddress
        fields = AddressSerializer.Meta.fields + ['addressType']


class BillingAddressSerializer(AddressSerializer):
    country = CountrySerializer(many=False)

    class Meta:
        model = BillingAddress
        fields = AddressSerializer.Meta.fields


class BillingAddressReadSerializer(AddressSerializer):
    country = CountrySerializer(many=False)
    city = CitySerializer(many=False)

    class Meta:
        model = BillingAddress
        fields = AddressSerializer.Meta.fields


class SupplierCategorySerializer(serializers.ModelSerializer):

    class Meta:
        model = SupplierCategory
        fields = ['id', 'name']


class SupplierSerializer(serializers.ModelSerializer):
    profilePicture = Base64ImageField(
        max_length=None, use_url=True, required=False
    )

    class Meta:
        model = Supplier
        fields = ['id', 'name', 'balance', 'email', 'supplierCategories', 'profilePicture',
                  'phoneNumber', 'description', 'notes', 'website', 'accountType',
                  'belongsTo', 'payableAccount', 'creditLimit']


class SupplierReadSerializer(SupplierSerializer):
    addresses = AddressReadSerializer(many=True)
    supplierCategories = SupplierCategorySerializer(many=True)
    contacts = SupplierContactSerializer(many=True)

    class Meta:
        model = Supplier
        fields = SupplierSerializer.Meta.fields + ['addresses', 'contacts']


class CertificationEntityReadSerializer(CertificationEntitySerializer):
    profilePicture = Base64ImageField(
        max_length=None, use_url=True, required=False
    )

    class Meta:
        model = CertificationEntity
        fields = CertificationEntitySerializer.Meta.fields + ['addresses']


class ConsumerClientSerializer(serializers.ModelSerializer):
    profilePicture = Base64ImageField(
        max_length=None, use_url=True, required=False
    )

    class Meta:
        model = ConsumerClient
        fields = ['id', 'profilePicture', 'createdBy', 'createdOn',
                  'isActive', 'balance', 'notes', 'name', 'email',
                  'phoneNumber', 'age', 'sex', 'accountType',
                  'belongsTo', 'receivableAccount', 'creditLimit', 'clientCategories', 'dateOfBirth']


class BusinessClientSerializer(serializers.ModelSerializer):
    profilePicture = Base64ImageField(
        max_length=None, use_url=True, required=False
    )

    class Meta:
        model = BusinessClient
        fields = ['id', 'profilePicture', 'createdBy', 'createdOn',
                  'isActive', 'balance', 'notes', 'name', 'email',
                  'phoneNumber', 'bio', 'website', 'accountType',
                  'belongsTo', 'receivableAccount', 'creditLimit', 'clientCategories']


class BusinessClientContactReadSerializer(BusinessClientContactSerializer):
    account = BusinessClientSerializer(many=False)
    title = TitleSerializer(many=False)

    class Meta:
        model = BusinessClientContact
        fields = BusinessClientContactSerializer.Meta.fields + ['fullName']


class SupplierContactReadSerializer(SupplierContactSerializer):
    account = SupplierReadSerializer(many=False)
    title = TitleSerializer(many=False)

    class Meta:
        model = SupplierContact
        fields = SupplierContactSerializer.Meta.fields + ['fullName']


class ContactReadSerializer(ContactSerializer):
    title = TitleSerializer(many=False)

    class Meta:
        model = Contact
        fields = ContactSerializer.Meta.fields + ['fullName']


class BusinessClientReadSerializer(BusinessClientSerializer):
    addresses = AddressReadSerializer(many=True)
    contacts = BusinessClientContactReadSerializer(many=True)

    class Meta:
        model = BusinessClient
        fields = BusinessClientSerializer.Meta.fields + \
            ['addresses', 'contacts']


class ConsumerClientReadSerializer(ConsumerClientSerializer):
    addresses = AddressReadSerializer(many=True)

    class Meta:
        model = ConsumerClient
        fields = ConsumerClientSerializer.Meta.fields + ['addresses']


class AllClientSerializer(serializers.Serializer):
    """ Serializer that renders each client with its own specific serializer """

    @classmethod
    def get_serializer(cls, model):
        if model == ConsumerClient:
            return ConsumerClientReadSerializer
        elif model == BusinessClient:
            return BusinessClientReadSerializer
        elif model == Account:
            return AccountReadSerializer

    def to_representation(self, instance):
        serializer = self.get_serializer(instance.__class__)
        return serializer(instance, context=self.context).data


class SpecificAccountByAccountTypeSerializer(serializers.Serializer):
    """ Serializer that renders each account with its own specific serializer """

    def to_representation(self, value):
        if value.accountType == AccountTypeChoices.BUSINESSCLIENT:
            account = BusinessClient.objects.get(id=value.id)
            serializer = BusinessClientReadSerializer(
                account, context=self.context)

        elif value.accountType == AccountTypeChoices.CONSUMERCLIENT:
            account = ConsumerClient.objects.get(id=value.id)
            serializer = ConsumerClientReadSerializer(
                account, context=self.context)
        elif value.accountType == AccountTypeChoices.COMPANY:
            account = Company.objects.get(id=value.id)
            serializer = CompanyReadSerializer(account, context=self.context)
        elif value.accountType == AccountTypeChoices.SUPPLIER:
            account = Supplier.objects.get(id=value.id)
            serializer = SupplierReadSerializer(account, context=self.context)
        else:
            raise Exception('Unexpected type of account object')

        return serializer.data


class LeadSerializer(serializers.ModelSerializer):

    class Meta:
        model = Lead
        fields = ['id', 'createdBy', 'createdOn',
                  'email', 'phoneNumber', 'status', 'source', 'website',
                  'description', 'assignedTo', 'opportunityAmount',
                  'account', 'firstName', 'lastName', 'accountType', 'isActive', 'addresses']


class LeadReadSerializer(LeadSerializer):
    addresses = AddressReadSerializer(many=True)

    class Meta:
        model = Lead
        fields = LeadSerializer.Meta.fields


class CompanyLocationSerializer(serializers.ModelSerializer):

    class Meta:
        model = CompanyLocation
        fields = ['id', 'address', 'title', 'email', 'phoneNumber', 'image']


class CompanyLocationReadSerializer(CompanyLocationSerializer):
    address = AddressReadSerializer(many=False)

    class Meta:
        model = CompanyLocation
        fields = CompanyLocationSerializer.Meta.fields


# class ResellerStoreSerializer(CompanyLocationSerializer):

#     class Meta:
#         model = CompanyLocation
#         fields = CompanyLocationSerializer.Meta.fields + ['businessClient']


# class ResellerStoreReadSerializer(ResellerStoreSerializer):
#     address = AddressReadSerializer(many=False)

#     class Meta:
#         model = CompanyLocation
#         fields = ResellerStoreSerializer.Meta.fields

class TestAccountSerializer(serializers.ModelSerializer):
    class Meta:
        model = TestAccount
        fields = ['id', 'createdOn',
                  'balance', 'notes', 'isActive', 'name']



from django.db import models
from .enums import SexChoice, AddressTypeChoices, AccountTypeChoices
from .choices import (STATUS_CHOICE, LEAD_STATUS, LEAD_SOURCE)
from .managers import (ContactManager, ConsumerClientManager)

class City(models.Model):
    name = models.CharField(max_length=100, null=True, blank=True)
    code = models.CharField(blank=True, max_length=3, null=True)

    class Meta:
        verbose_name = 'City'
        verbose_name_plural = 'Cities'

    def __str__(self):
        return self.name

class Country(models.Model):
    name = models.CharField(max_length=60, null=True, blank=True)
    code = models.CharField(blank=True, max_length=3, null=True)

    class Meta:
        verbose_name = 'Country'
        verbose_name_plural = 'Countries'

    def __str__(self):
        return self.name + ' (' + self.code + ')'

class TestAccount(models.Model):
    createdOn = models.DateTimeField(auto_now_add=True)
    isActive = models.BooleanField(default=False)
    balance = models.DecimalField(decimal_places=2, max_digits=10, null=True, default=0)
    notes = models.TextField(blank=True, max_length=1000, default='')
    name = models.CharField(blank=False, max_length=60, null=True)

    class Meta:
        verbose_name = 'Test Account'
        verbose_name_plural = 'Test Accounts'

    def __str__(self):
        return self.name

class Account(models.Model):
    profilePicture = models.ImageField(upload_to='accounts-profile-pictures',
                                       blank=True, default="defaultImage.jpg", null=True)
    createdBy = models.ForeignKey('team_common.User', on_delete=models.SET_NULL, null=True, related_name="accountsCreatedByMe")
    createdOn = models.DateTimeField(auto_now_add=True)
    isActive = models.BooleanField(default=False)
    balance = models.DecimalField(decimal_places=2, max_digits=10, null=True, default=0)
    notes = models.TextField(blank=True, max_length=1000, default='')
    accountType = models.PositiveSmallIntegerField(choices=AccountTypeChoices.choices, null=True, blank=True)
    name = models.CharField(blank=False, max_length=60, null=True)
    customerId = models.CharField(blank=True, max_length=200, null=True)
    # refactor... add phonenumber here.

    class Meta:
        verbose_name = 'Account'
        verbose_name_plural = 'Accounts'

    def __str__(self):
        return self.name

class Address(models.Model):
    name = models.CharField(blank=False, max_length=60, null=True)
    addressLine1 = models.CharField(blank=False, max_length=60)
    addressLine2 = models.CharField(blank=True, max_length=60)
    region = models.CharField("State / Province / Region", blank=False, max_length=60)
    postalCode = models.CharField("ZIP / Postal code", max_length=12)
    city = models.ForeignKey(City, blank=True, null=True, on_delete=models.SET_NULL)
    country = models.ForeignKey(Country, blank=True, null=True, on_delete=models.SET_NULL)
    account = models.ForeignKey(Account, blank=True, null=True, on_delete=models.SET_NULL,
                                related_name="addresses")

    @property
    def value(self):
        if self.addressLine2 is None or not self.addressLine2:
            return self.addressLine1 + ', ' + self.city.name + ', ' + self.region + ', ' + self.country.name + ', ' + self.postalCode
        else:
            return self.addressLine1 + ', ' + self.addressLine2 + ', ' + self.city.name + ', ' + self.region + ', ' + self.country.name + ', ' + self.postalCode

    class Meta:
        verbose_name = 'Address'
        verbose_name_plural = 'Addresses'

    def __str__(self):
        if self.addressLine2 is None:
            return self.addressLine1 + ', ' + self.city.name + ', ' + self.region + ', ' + self.country.name + ', ' + self.postalCode
        else:
            return self.addressLine1 + ', ' + self.addressLine2 + ', ' + self.city.name + ', ' + self.region + ', ' + self.country.name + ', ' + self.postalCode


class ShippingAddress(Address):
    addressType = models.CharField(max_length=10, choices=AddressTypeChoices.choices(),
                                   default=AddressTypeChoices.SHIPPING)

    class Meta:
        verbose_name = 'Shipping Address'
        verbose_name_plural = 'Shipping Addresses'

    def __str__(self):
        return "Shipping Address " + str(self.id)


class BillingAddress(Address):
    addressType = models.CharField(max_length=10, choices=AddressTypeChoices.choices(),
                                   default=AddressTypeChoices.BILLING)

    class Meta:
        verbose_name = 'Billing Address'
        verbose_name_plural = 'Billing Addresses'

    def __str__(self):
        return "Billing Address " + str(self.id)


class Company(Account):
    email = models.EmailField(blank=True, max_length=254, unique=True, null=True)
    phoneNumber = models.CharField(blank=True, max_length=20, null=True)
    website = models.URLField(max_length=500, blank=True, null=True)
    plan = models.ForeignKey('plans_shared.Plan', null=True, blank=True, related_name="companies",
                             on_delete=models.SET_NULL)
    businessClient = models.OneToOneField('accounts_shared.BusinessClient', related_name='company',
                                on_delete=models.SET_NULL, blank=True, null=True)
    
    # def get_company(request):
    #     company = request.user.employee.company
    #     return company

    class Meta:
        verbose_name = 'Company'
        verbose_name_plural = 'Companies'

    def __str__(self):
        return "Company " + str(self.id)

class ClientCategory(models.Model):
    name = models.CharField(blank=False, max_length=60, null=False)

    class Meta:
        verbose_name = 'Client Category'
        verbose_name_plural = 'Client Categories'

    def __str__(self):
        return self.name


class BusinessAccount(Account):
    email = models.EmailField(blank=False, max_length=254)
    phoneNumber = models.CharField(blank=False, max_length=20)
    website = models.URLField(max_length=500, blank=True, null=True)
    belongsTo = models.ForeignKey(Company,related_name='businessAccounts',
                                on_delete=models.SET_NULL, blank=True, null=True)

    class Meta:
        verbose_name = 'Business Account'
        verbose_name_plural = 'Business Accounts'

    def __str__(self):
        return self.name


class Contact(models.Model):
    objects = ContactManager()
    DEFAULT_PICTURE = "../media/defaultImage.jpg"
    firstName = models.CharField(max_length=50)
    lastName = models.CharField(max_length=50)
    email = models.EmailField()
    phoneNumber = models.CharField(max_length=20)
    description = models.TextField(blank=True, null=True)
    createdOn = models.DateTimeField(auto_now_add=True)
    isActive = models.BooleanField(default=False)
    profilePicture = models.ImageField(upload_to='contact-profile-pictures',
                                       blank=True, default=DEFAULT_PICTURE, null=True)
    title = models.ForeignKey(
        'team_common.Title', on_delete=models.CASCADE, null=True, blank=True)
    
    @property
    def fullName(self):
        return self.firstName + ' ' + self.lastName

    class Meta:
        verbose_name = 'Contact'
        verbose_name_plural = 'Contacts'

    def __str__(self):
        return self.firstName + self.lastName


class SupplierCategory(models.Model):
    name = models.CharField(blank=False, max_length=60)

    class Meta:
        verbose_name = 'Supplier Category'
        verbose_name_plural = 'Supplier Categories'

    def __str__(self):
        return self.name


class Supplier(BusinessAccount):
    description = models.TextField(blank=True, max_length=1000, default='')
    supplierCategories = models.ManyToManyField(SupplierCategory, related_name="suppliers", blank=True)
    payableAccount = models.ForeignKey('accounting_shared.ChartAccount', null=True, blank=True, on_delete=models.SET_NULL)
    creditLimit = models.OneToOneField('accounting_shared.CreditLimit', null=True, blank=True,
                                    on_delete=models.SET_NULL, related_name="supplier")

    class Meta:
        verbose_name = 'Supplier'
        verbose_name_plural = 'Suppliers'

    def __str__(self):
        return self.name


class CertificationEntity(BusinessAccount):
    code = models.CharField(blank=False, max_length=30)
    description = models.TextField(blank=True, max_length=1000, default='')

    class Meta:
        verbose_name = 'CertificationEntity'
        verbose_name_plural = 'CertificationEntities'

    def __str__(self):
        return self.name

class BusinessClient(BusinessAccount):
    bio = models.TextField(blank=True, max_length=1000, default='')
    isReseller = models.BooleanField(default=False)
    receivableAccount = models.ForeignKey('accounting_shared.ChartAccount', null=True, blank=True, on_delete=models.SET_NULL)
    creditLimit = models.OneToOneField('accounting_shared.CreditLimit', null=True, blank=True,
                                    on_delete=models.SET_NULL, related_name="businessClient")
    clientCategories = models.ManyToManyField(ClientCategory, related_name="businessClients", blank=True)

    class Meta:
        verbose_name = 'Business Client'
        verbose_name_plural = 'Business Clients'

class SupplierContact(Contact):
    account = models.ForeignKey(Supplier, related_name='contacts',
                                on_delete=models.SET_NULL, blank=True, null=True)

    class Meta:
        verbose_name = 'Supplier Contact'
        verbose_name_plural = 'Supplier Contacts'

    def __str__(self):
        return "Suipplier Contact " + str(self.id)


class BusinessClientContact(Contact):
    account = models.ForeignKey(BusinessClient, related_name='contacts',
                                on_delete=models.SET_NULL, blank=True, null=True)
    user = models.OneToOneField('team_common.User', related_name='contact',
                                on_delete=models.SET_NULL, blank=True, null=True)

    class Meta:
        verbose_name = 'Business Client Contact'
        verbose_name_plural = 'Business Client Contacts'

    def __str__(self):
        if self.firstName and self.lastName:
            return self.firstName + " " + self.lastName
        else:
            return "Business Client Contact " + str(self.id)


class ConsumerClient(Account):
    objects = ConsumerClientManager()
    email = models.EmailField(blank=False, max_length=254)
    phoneNumber = models.CharField(blank=False, max_length=20)
    age = models.PositiveIntegerField(default=0)
    dateOfBirth = models.DateField(null=True, blank=True)
    sex = models.CharField(max_length=10, choices=SexChoice.choices())
    user = models.OneToOneField('team_common.User', on_delete=models.SET_NULL, null=True, blank=True, related_name="account")
    belongsTo = models.ForeignKey(Company,related_name='consumerClients',
                                on_delete=models.SET_NULL, blank=True, null=True)
    receivableAccount = models.ForeignKey('accounting_shared.ChartAccount', null=True, blank=True, on_delete=models.SET_NULL)
    creditLimit = models.OneToOneField('accounting_shared.CreditLimit', null=True, blank=True,
                                    on_delete=models.SET_NULL, related_name="consumerClient")
    clientCategories = models.ManyToManyField(ClientCategory, related_name="consumerClients", blank=True)

    class Meta:
        verbose_name = 'Consumer Client'
        verbose_name_plural = 'Consumer Clients'
        unique_together = ('belongsTo', 'email')


class Lead(models.Model):
    firstName = models.CharField(null=False, blank=False, max_length=50)
    lastName = models.CharField(null=False, blank=False, max_length=50)
    email = models.EmailField(null=False, blank=False)
    phoneNumber = models.CharField(null=True, blank=True, max_length=50)
    status = models.CharField(max_length=255, blank=True, null=True, choices=LEAD_STATUS
                              )
    source = models.CharField(max_length=255, blank=True, null=True, choices=LEAD_SOURCE)
    addresses = models.ManyToManyField(Address, related_name="leads")
    website = models.CharField(max_length=255, blank=True, null=True)
    description = models.TextField(blank=True, null=True)
    assignedTo = models.ManyToManyField('team.Employee', related_name="myLeads")
    opportunityAmount = models.DecimalField(decimal_places=2, max_digits=12, blank=True, null=True
                                            )
    createdBy = models.ForeignKey('team.Employee', on_delete=models.SET_NULL, null=True)
    createdOn = models.DateTimeField(auto_now_add=True)
    isActive = models.BooleanField(default=False)
    # enquery_type = models.CharField(max_length=255, blank=True, null=True)
    # tags = models.ManyToManyField(Tags, blank=True)
    # contacts = models.ManyToManyField(Contact, related_name="contacts")
    # created_from_site = models.BooleanField(default=False)
    # teams = models.ManyToManyField(Teams, related_name="lead_teams")
    # account = models.ForeignKey(
    #     Account, on_delete=models.SET_NULL, null=True, blank=True
    # )

    class Meta:
        ordering = ["-createdOn"]
        verbose_name = 'Lead'
        verbose_name_plural = 'Leads'

    def __str__(self):
        return self.email

# TODO: add hours of CompanyLocation

class CompanyLocation(models.Model):
    DEFAULT_PICTURE = "../media/defaultImage.jpg"

    email = models.EmailField(blank=False, max_length=254)
    phoneNumber = models.CharField(blank=False, max_length=20)
    title = models.CharField(blank=False, null=False, max_length=200)
    address = models.ForeignKey(Address, on_delete=models.SET_NULL, null=True, blank=True)
    image = models.ImageField(upload_to='company-location-pictures',
                              blank=True, default=DEFAULT_PICTURE, null=True)

    class Meta:
        verbose_name = 'Company Location'
        verbose_name_plural = 'Company Locations'

    def __str__(self):
        return self.title

class Store(models.Model):
    DEFAULT_PICTURE = "../media/defaultImage.jpg"

    email = models.EmailField(blank=False, max_length=254)
    phoneNumber = models.CharField(blank=False, max_length=20)
    title = models.CharField(blank=False, null=False, max_length=200)
    # address = models.ForeignKey(Address, on_delete=models.SET_NULL, null=True, blank=True)
    image = models.ImageField(upload_to='store-pictures',
                              blank=True, default=DEFAULT_PICTURE, null=True)

    class Meta:
        verbose_name = 'Store'
        verbose_name_plural = 'Stores'

    def __str__(self):
        return self.title








# Generated by Django 4.0.5 on 2022-06-23 14:01

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('accounting_shared', '0001_initial'),
        ('accounts_shared', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='businessclient',
            name='creditLimit',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='businessClient', to='accounting_shared.creditlimit'),
        ),
        migrations.AddField(
            model_name='businessclient',
            name='receivableAccount',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='accounting_shared.chartaccount'),
        ),
        migrations.AddField(
            model_name='consumerclient',
            name='creditLimit',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='consumerClient', to='accounting_shared.creditlimit'),
        ),
        migrations.AddField(
            model_name='consumerclient',
            name='receivableAccount',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='accounting_shared.chartaccount'),
        ),
        migrations.AddField(
            model_name='supplier',
            name='creditLimit',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='supplier', to='accounting_shared.creditlimit'),
        ),
        migrations.AddField(
            model_name='supplier',
            name='payableAccount',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='accounting_shared.chartaccount'),
        ),
    ]
